<?php
//
//	Module name      :	dateutil.php
//	Version          :	1.0
//	Started on       :	April 29, 2005
//	Completed on     :	Maj 7, 2007
//
//	Module type      :	Library
//	Description      :	Functions for date and time management
//
//	Developed under  :	Php 4.1.2
//	Sw requirements  :	None
//
//	Parameters       :	None
//	Notes - usage    :	Month and days names are stored in Italian language; see $DU_month_names or
//						$DU_day_names to change them.
//


//--- Include files --------------------------------------------------------------------------------



//--- Public constants -----------------------------------------------------------------------------



define("DU_MODULE_NAME", "dateutil.php 1.0; on Maj 7, 2007 by J.Hubb.");

// Some format for Iso dates
define("DU_ISO_DATE",		"YYYY-MM-DD");					// Only date
define("DU_ISO_DATETIME",	"YYYY-MM-DDThh:mm:ss");			// Date and time

// Mode constants for DUNormalizeYear
define("DU_TODAY_CENTER",	0);		// Make current year the center of reference century (-50..+49)
define("DU_TODAY_FIRST",	1);		// Make current year the first one of reference century (0..+99)
define("DU_TODAY_LAST",		2);		// Make current year the last one of reference century (-99..0)
define("DU_FROM_YEAR",		3);		// Make specified year the first one of reference century
define("DU_TO_YEAR",		4);		// Make specified year the last one of reference century

// Lenghts in seconds
define("DU_MIN_SECS",	60);
define("DU_HOUR_SECS",	3600);
define("DU_DAY_SECS",	86400);
define("DU_WEEK_SECS",	604800);
define("DU_YEAR_SECS",	31557600);	// Medium year, 365.25 days

// Lenghts in minutes
define("DU_HOUR_MINS",	60);
define("DU_DAY_MINS",	1440);
define("DU_WEEK_MINS",	10080);
define("DU_YEAR_MINS",	525960);	// Medium year, 365.25 days

// Lenghts in hours
define("DU_DAY_HOURS",	24);
define("DU_WEEK_HOURS",	168);
define("DU_YEAR_HOURS",	8766);		// Medium year, 365.25 days

// Lenghts in days
define("DU_WEEK_DAYS",	7);
define("DU_YEAR_DAYS",	365.25);	// Medium year, 365.25 days

// Error codes
define("DU_NO_ERROR",		0);
define("DU_OUT_OF_RANGE",	1);
define("DU_TIMER_ERROR",	2);
define("DU_LAST_ERROR",		2);



//--- Public variables -----------------------------------------------------------------------------



$DU_month_names = array(
		1 => "Gennaio", "Febbraio", "Marzo", "Aprile", "Maggio", "Giugno",
		"Luglio", "Agosto", "Settembre", "Ottobre", "Novembre", "Dicembre");

$DU_day_names = array(
		0 => "Domenica", "Lunedi`", "Martedi`", "Mercoledi`", "Giovedi`", "Venerdi`", "Sabato");



//--- Private constants ----------------------------------------------------------------------------



$_DU_timer = array();		// Timer timestamps;



//--- Private variables ----------------------------------------------------------------------------



$_DU_error_num = DU_NO_ERROR;
$_DU_error_msg = array(
	DU_NO_ERROR		=> "No error",
	DU_OUT_OF_RANGE	=> "Out of range",
	DU_TIMER_ERROR	=> "Timer never reset (started)"
	);



//--- Public functions code ------------------------------------------------------------------------



function DUDayName($d)
// Returns the name of the day number $d (0 and 7 stand for Sunday); if $d is outside the range 0..7,
// FALSE is returned and an error code is set.
{
	global $_DU_error_num, $DU_day_names;

	if ($d >= 0 && $d <= 7)
	{
		$_DU_error_num = DU_NO_ERROR;
		return $DU_day_names[$d % 7];
	}
	else
	{
		$_DU_error_num = DU_OUT_OF_RANGE;
		return FALSE;
	}
}



function DUDayShortName($d)
// Returns the short name of the day number $d (0 and 7 stand for Sunday); if $d is outside the range 0..7,
// FALSE is returned and an error code is set.
{
	return substr(DUDayName($d), 0, 3);
}



function DUDayNumOfMonth($m = 0, $y = 0)
// Returns the number of days in the month $m of the year $y; if parameters are omitted, the
// current year and/or month are used. The year must be after 1970, otherwise FALSE is returned
// and an error code is set.
{
	global $_DU_error_num;

	if ($y <> 0 && $y < 1970)
	{
		$_DU_error_num = DU_OUT_OF_RANGE;
		return FALSE;
	}
	$_DU_error_num = DU_NO_ERROR;
	return date("t", mktime(0, 0, 0, ($m == 0 ? date("n") : $m), 1, ($y == 0 ? date("Y") : $y)));
}



function DUDayNumOfYear($y = 0)
// Returns the number of days of the year $y; if $y is omitted, the current year is used. The year
// must be after 1970, otherwise FALSE is returned and an error code is set.
{
	global $_DU_error_num;

	if ($y <> 0 && $y < 1970)
	{
		$_DU_error_num = DU_OUT_OF_RANGE;
		return FALSE;
	}
	$_DU_error_num = DU_NO_ERROR;
	return date("L", mktime(0, 0, 0, 1, 1, ($y == 0 ? date("Y") : $y))) ? 366 : 365;
}



function DUErrorMsg()
// Returns the error message of the last module function called.
{
	global $_DU_error_msg, $_DU_error_num;

	return $_DU_error_msg[$_DU_error_num];
}



function DUIso2Ts($iso)
// Returns the timestamp relative to the date string $iso, with Iso format ("YYYY-MM-DDThh:mm:ss.sTZD", where
// YYYY = year, MM = month, DD = day of month, hh = hour, mm = minute, ss = second, s = decimal fraction of
// a second, TZD = (Z | (+|-)hh:mm); the format can be truncated after any field, s and TZD are ignored);
// if $iso is not in the right format, the result is unpredictable. Does not affect error code.
{
	$y = (integer) substr($iso, 0, 4);
	$iso = substr($iso, 5);
	$m = (integer) substr($iso, 0, 2);
	$iso = substr($iso, 3);
	$d = (integer) substr($iso, 0, 2);
	$iso = substr($iso, 3);
	$hh = (integer) substr($iso, 0, 2);
	$iso = substr($iso, 3);
	$mm = (integer) substr($iso, 0, 2);
	$iso = substr($iso, 3);
	$ss = (integer) substr($iso, 0, 2);
	return mktime($hh, $mm, $ss, ($m == 0 ? 1 : $m), ($d == 0 ? 1 : $d), ($y < 1970 ? 1970 : $y));
}



function DUMonthName($m)
// Returns the name of the month number $m; if $m is outside the range 1..12, FALSE is returned
// and an error code is set.
{
	global $_DU_error_num, $DU_month_names;

	if ($m > 0 && $m < 13)
	{
		$_DU_error_num = DU_NO_ERROR;
		return $DU_month_names[$m];
	}
	else
	{
		$_DU_error_num = DU_OUT_OF_RANGE;
		return FALSE;
	}
}



function DUMonthShortName($m)
// Returns the short name of the month number $m; if $m is outside the range 1..12, FALSE is returned
// and an error code is set.
{
	return substr(DUMonthName($m), 0, 3);
}



function DUNormalizeYear($y, $mode = DU_TODAY_CENTER, $y2 = 0, $curr_yyyy = FALSE)
// Returns a four digits year for the specified year $y; if $y is a two digits year, then it is normalized
// according to $mode (see constants section) and according to $y2 when needed. The current four digits
// year $curr_yyyy is calculated or it can be passed for tests.
{
	if ($y < 100)
	{
		if (($curr_yyyy === FALSE) || ($curr_yyyy < 100))
			$curr_yyyy = date("Y");
		switch ($mode)
		{
			case DU_TODAY_FIRST:
				$low = $curr_yyyy;
				break;
			case DU_TODAY_LAST:
				$low = $curr_yyyy - 99;
				break;
			case DU_FROM_YEAR:
				$low = ($y2 < 100 ? date("Y") : $y2);
				break;
			case DU_TO_YEAR:
				$low = ($y2 < 100 ? date("Y") : $y2) - 99;
				break;
			default:
				$low = $curr_yyyy - 50;
				break;
		}
		$low_cent = substr($low, 0, 2) . "00";
		$y += $low_cent;
		if (($y < $low) || ($y > ($low + 99)))
			$y += 100;
	}
	return $y;
}



function DUResult()
// Returns the error code of the last module function called.
{
	global $_DU_error_num;

	return $_DU_error_num;
}



function DUTimerReset($name)
// Resets (or creates) the timer with name $name.
{
	global $_DU_timer, $_DU_error_num;

	$_DU_error_num = DU_NO_ERROR;
	$_DU_timer[$name] = microtime();
}



function DUTimerRead($name)
// Returns elapsed seconds (with microsecond resolution) since the reset of the timer named $name.
// If $name does not exist (it has never been reset), it resets it and returns 0.
{
	global $_DU_timer, $_DU_error_num;

	$now = microtime();
	$t1 = explode(" ", $now);
	if (array_key_exists($name, $_DU_timer))
	{
		$t0 = explode(" ", $_DU_timer[$name]);
		$_DU_error_num = DU_NO_ERROR;
	}
	else
	{
		$t0 = $t1;
		DUTimerReset($name);
		$_DU_error_num = DU_TIMER_ERROR;
	}
	return number_format($t1[1] - $t0[1] + $t1[0] - $t0[0], 6);
}



function DUTsExplode($ts)
// For the timestamp $ts, returns the corresponding associative array with indexes "Y", "M", "D", "h", "m",
// "s", "wd" (week day) with numerical values and "0M", "0D", "0h", "0m", "0s" with 0 padded string values.
// Does not affect error code.
{
	return array(
			"Y" => (integer) date("Y", $ts), "M" => (integer) date("n", $ts), "D" => (integer) date("j", $ts),
			"h" => (integer) date("G", $ts), "m" => (integer) date("i", $ts), "s" => (integer) date("s", $ts),
			"wd" => (integer) date("w", $ts),
			"0M" => date("m", $ts), "0D" => date("d", $ts),
			"0h" => date("H", $ts), "0m" => date("i", $ts), "0s" => date("s", $ts)
			);
}



function DUTsImplode($t)
// Converts the associative array $t, with indexes "Y", "M", "D", "h", "m", "s" to the corresponding
// timestamp. Does not affect error code.
{
	if (! is_array($t))
		return 0;
	else
		return mktime(
				array_key_exists("h", $t) ? $t["h"] : 0,
				array_key_exists("m", $t) ? $t["m"] : 0,
				array_key_exists("s", $t) ? $t["s"] : 0,
				array_key_exists("M", $t) ? $t["M"] : 1,
				array_key_exists("D", $t) ? $t["D"] : 1,
				array_key_exists("Y", $t) ? $t["Y"] : 1970
				);
}



function DUTs2Iso($ts, $fmt = DU_ISO_DATETIME)
// For the timestamp $ts, returns the corresponding string in Iso format ("YYYY-MM-DDThh:mm:ss.sTZD", where
// YYYY = year, MM = month, DD = day of month, hh = hour, mm = minute, ss = second, s = decimal fraction of
// a second, TZD = (Z | (+|-)hh:mm); s and TZD are always omitted); the format can be truncated specifying
// $fmt parameter and using defined constants. Does not affect error code.
{
	switch (strlen($fmt))
	{
		case 4:
			return date("Y", $ts);
		case 7:
			return date("Y-m", $ts);
		case 10:
			return date("Y-m-d", $ts);
		case 13:
			return date("Y-m-d\Th", $ts);
		case 16:
			return date("Y-m-d\Th:i", $ts);
		default:
			return date("Y-m-d\Th:i:s", $ts);
	}
}



//--- Private functions code -----------------------------------------------------------------------



//--- Initialization code --------------------------------------------------------------------------



?>
